// Wave.cpp: implementation of the CWave class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <mmsystem.h>

#include "Wave.h"

#pragma comment(lib, "winmm.lib")

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWave::CWave()
{
	m_nType = 0;
	m_bRecording = FALSE;
	m_bOpened = FALSE;
	m_pSoundData = NULL;
	m_wDeviceID = 0;
}

CWave::~CWave()
{
	Close ();
}

int CWave::GetDevices (void)
{
	return waveOutGetNumDevs ();
}

BOOL CWave::LoadFromDisk (CString Filename)
{
	Close ();

	CFile File;
	if (!File.Open (Filename, CFile::modeRead))
		return FALSE;

	DWORD dwLength = File.GetLength ();

	m_pSoundData = AllocMem (dwLength);
	if (!m_pSoundData){
		File.Close ();
		return FALSE;
		}

//	if (File.ReadHuge (m_pSoundData, dwLength) != dwLength){
//		File.Close ();
//		return FALSE;
//		}

	File.Close ();

	m_bOpened = TRUE;
	m_nType = 1;

	return TRUE;
}

BOOL CWave::LoadFromRes (CString ResID, HINSTANCE hInstance)
{
	Close ();

	m_hInstance = hInstance;

	HANDLE hResInfo;
	hResInfo = FindResource (m_hInstance, ResID, "WAVE");
	if (!hResInfo) 
		return FALSE;

	m_hResHandle = LoadResource (m_hInstance, (HRSRC) hResInfo);
	if (!m_hResHandle) 
		return FALSE;

	m_pSoundData = LockResource (m_hResHandle);
	if (!m_pSoundData)
		return FALSE;

	m_bOpened = TRUE;
	m_nType = 2;

	return TRUE;
}

void CWave::Close (void)
{
	PlaySound (NULL, NULL, NULL);

	if (m_pSoundData){
		if (m_nType == 1)
			FreeMem (m_pSoundData);
		if (m_nType == 2){
			UnlockResource (m_hResHandle);
			FreeResource (m_hResHandle);
			}
		m_pSoundData = NULL;
		}

	m_nType = 0;
	m_bRecording = FALSE;
	m_bOpened = FALSE;

	if (m_wDeviceID){
		mciSendCommand (m_wDeviceID, MCI_STOP, MCI_WAIT, NULL);
		mciSendCommand (m_wDeviceID, MCI_CLOSE, NULL, NULL);
		}
	m_wDeviceID = 0;
}

BOOL CWave::Play (BOOL bLoop)
{
	if (m_bOpened){
		if (m_bRecording){
			Stop ();
			MCI_PLAY_PARMS PlayParms;
			mciSendCommand (m_wDeviceID, MCI_SEEK, MCI_SEEK_TO_START, NULL);
			if (mciSendCommand (m_wDeviceID, MCI_PLAY, NULL, (DWORD)(LPVOID) &PlayParms))
				return FALSE;
			else
				return TRUE;
			}
		else{
			DWORD dwFlags = SND_MEMORY | SND_ASYNC | SND_NODEFAULT;
			if (bLoop)
				dwFlags |= SND_LOOP;

			return PlaySound ((char *) m_pSoundData, NULL, dwFlags);
			}
		}

	return FALSE;
}

void CWave::Stop (void)
{
	if (m_bRecording)
		mciSendCommand (m_wDeviceID, MCI_STOP, NULL, NULL);
	else
		PlaySound (NULL, NULL, NULL);
}
	
BOOL CWave::PlayFromDisk (CString Filename)
{
	return PlaySound (Filename, NULL, SND_FILENAME | SND_SYNC | SND_NODEFAULT);
}

BOOL CWave::PlayFromRes (CString ResID, HINSTANCE hInstance)
{
	return PlaySound (ResID, hInstance, SND_RESOURCE | SND_SYNC | SND_NODEFAULT);
}

void *CWave::AllocMem (DWORD dwSize)
{
	HGLOBAL Global;
	HGLOBAL *pGlobal;
	unsigned char *pMem;

	Global = GlobalAlloc (GMEM_FIXED, (DWORD) dwSize + 10);
	if (!Global) return (NULL);

	pMem = (unsigned char *) GlobalLock (Global);
	pGlobal = (HGLOBAL *) pMem;
	*pGlobal = Global;
	pMem += sizeof (HGLOBAL);

	return (pMem);

}

void CWave::FreeMem (void far *pMem)
{
	HGLOBAL Global;
	HGLOBAL *pGlobal;
	unsigned char *pUChar;

	pUChar = (unsigned char *) pMem;
	pUChar -= sizeof (HGLOBAL);

	pGlobal = (HGLOBAL *) pUChar;
	Global = *pGlobal;

	GlobalUnlock (Global);
	GlobalFree (Global);
}
